      SUBROUTINE PDGESVD4( M, N, A, IA, JA, DESCA, S,
     $                     WORK, LWORK, INFO )
*
* Scalar Arguments
      INTEGER            IA, INFO, JA, LWORK, M, N
*
* Array Arguments
      INTEGER            DESCA( * )
      DOUBLE PRECISION   A( * ), S( * ), WORK( * )
*
* Parameters
      INTEGER            DLEN_, CTXT_,
     $                   MB_, NB_, RSRC_, CSRC_, ITHVAL
      PARAMETER          ( DLEN_ = 9, CTXT_ = 2, MB_ = 5, NB_ = 6,
     $                   RSRC_ = 7, CSRC_ = 8, ITHVAL = 10 )
      DOUBLE PRECISION   ZERO, ONE
      PARAMETER          ( ZERO = 0.0D+0, ONE = 1.0D+0 )
*
* Local Scalars
      CHARACTER          UPLO
      INTEGER            I, INDD, INDD2, INDE,
     $                   INDE2, INDTAUP, INDTAUQ, INDWORK,
     $                   IOFFD, IOFFE, ISCALE, J, K, LLWORK,
     $                   LWMIN, MAXIM, MB, MP, MYPCOL,
     $                   MYPROW, NB, NPCOL, NPROCS, NPROW,
     $                   NQ, SIZE, SIZEB, SIZEPOS,
     $                   WATOBD, WBDTOSVD, WDBDSQR, WPDGEBRD, WPDLANGE
      DOUBLE PRECISION   ANRM, BIGNUM, EPS, RMAX, RMIN, SAFMIN, SIGMA,
     $                   SMLNUM
*
* Local Arrays
      INTEGER            IDUM1( 1 ), IDUM2( 1 )
      DOUBLE PRECISION   C( 1, 1 )
*
* External Functions
      LOGICAL            LSAME
      INTEGER            NUMROC
      DOUBLE PRECISION   PDLAMCH, PDLANGE
      EXTERNAL           LSAME, NUMROC, PDLAMCH, PDLANGE
*
* External Subroutines
      EXTERNAL           BLACS_GRIDINFO, CHK1MAT, DBDSQR,
     $                   DGAMN2D, DGAMX2D, DSCAL, IGAMX2D, IGEBR2D,
     $                   IGEBS2D, PCHK1MAT, PDGEBRD,
     $                   PDLARED1D, PDLARED2D, PDLASCL, PXERBLA
*
* Intrinsic Functions
      INTRINSIC          DBLE, MAX, MIN, SQRT
*
* Executable Statements
*
      CALL BLACS_GRIDINFO( DESCA( CTXT_ ), NPROW, NPCOL, MYPROW,
     $                     MYPCOL )
      ISCALE = 0
      INFO = 0
*
      IF( NPROW.EQ.-1 ) THEN
         INFO = -( 600+CTXT_ )
      ELSE
*
         SIZE = MIN( M, N )
         SIZEB = MAX( M, N )
         NPROCS = NPROW*NPCOL
         IF( M.GE.N ) THEN
            IOFFD = JA - 1
            IOFFE = IA - 1
            SIZEPOS = 1
         ELSE
            IOFFD = IA - 1
            IOFFE = JA - 1
            SIZEPOS = 3
         END IF
            WANTU = 0
            WANTVT = 0
*
         CALL CHK1MAT( M, 1, N, 2, IA, JA, DESCA, 6, INFO )
         CALL IGAMX2D( DESCA( CTXT_ ), 'A', ' ', 1, 1, INFO, 1, 1, 1,
     $                 -1, -1, 0 )
*
         IF( INFO.EQ.0 ) THEN
*
* Set up pointers into the WORK array.
*
            INDD = 2
            INDE = INDD + SIZEB + IOFFD
            INDD2 = INDE + SIZEB + IOFFE
            INDE2 = INDD2 + SIZEB + IOFFD
            INDTAUQ = INDE2 + SIZEB + IOFFE
            INDTAUP = INDTAUQ + SIZEB + JA - 1
            INDWORK = INDTAUP + SIZEB + IA - 1
            LLWORK = LWORK - INDWORK + 1
*
* Set local dimensions of matrices (this is for MB=NB=1).
*
            NB = DESCA( NB_ )
            MB = DESCA( MB_ )
            MP = NUMROC( M, MB, MYPROW, DESCA( RSRC_ ), NPROW )
            NQ = NUMROC( N, NB, MYPCOL, DESCA( CSRC_ ), NPCOL )
*
* Transmit MAX(NQ0, MP0).
*
            IF( MYPROW.EQ.0 .AND. MYPCOL.EQ.0 ) THEN
               MAXIM = MAX( NQ, MP )
               CALL IGEBS2D( DESCA( CTXT_ ), 'All', ' ', 1, 1, MAXIM,
     $                       1 )
            ELSE
               CALL IGEBR2D( DESCA( CTXT_ ), 'All', ' ', 1, 1, MAXIM, 1,
     $                       0, 0 )
            END IF
*
            WPDLANGE = MP
            WPDGEBRD = NB*( MP+NQ+1 ) + NQ
            WATOBD = MAX( MAX( WPDLANGE, WPDGEBRD ), MAXIM )
*
            WDBDSQR = MAX( 1, 2*SIZE )
            WBDTOSVD =  MAX( WDBDSQR, 0)
*
* Finally, calculate required workspace.
*
            LWMIN = 2 + 6*SIZEB + MAX( WATOBD, WBDTOSVD )
            WORK( 1 ) = DBLE( LWMIN )
*
            IF( LWORK.LT.LWMIN .AND. LWORK.NE.-1 ) THEN
               INFO = -9
            END IF
*
         END IF
*
         IF( LWORK.EQ.-1 ) THEN
            IDUM1( 1 ) = -1
         ELSE
            IDUM1( 1 ) = 1
         END IF
         IDUM2( 1 ) = 9
         CALL PCHK1MAT( M, 1, N, 2, IA, JA, DESCA, 6, 1, IDUM1, IDUM2,
     $                  INFO )
*
      END IF
*
      IF( INFO.NE.0 ) THEN
         CALL PXERBLA( DESCA( CTXT_ ), 'PDGESVD4', -INFO )
         RETURN
      ELSE IF( LWORK.EQ.-1 ) THEN
         GO TO 40
      END IF
*
* Quick return if possible.
*
      IF( M.LE.0 .OR. N.LE.0 )
     $   GO TO 40
*
* Get machine constants.
*
      SAFMIN = PDLAMCH( DESCA( CTXT_ ), 'Safe minimum' )
      EPS = PDLAMCH( DESCA( CTXT_ ), 'Precision' )
      SMLNUM = SAFMIN / EPS
      BIGNUM = ONE / SMLNUM
      RMIN = SQRT( SMLNUM )
      RMAX = MIN( SQRT( BIGNUM ), ONE / SQRT( SQRT( SAFMIN ) ) )
*
* Scale matrix to allowable range, if necessary.
*
      ANRM = PDLANGE( '1', M, N, A, IA, JA, DESCA, WORK( INDWORK ) )
      IF( ANRM.GT.ZERO .AND. ANRM.LT.RMIN ) THEN
         ISCALE = 1
         SIGMA = RMIN / ANRM
      ELSE IF( ANRM.GT.RMAX ) THEN
         ISCALE = 1
         SIGMA = RMAX / ANRM
      END IF
*
      IF( ISCALE.EQ.1 ) THEN
         CALL PDLASCL( 'G', ONE, SIGMA, M, N, A, IA, JA, DESCA, INFO )
      END IF
*
      CALL PDGEBRD( M, N, A, IA, JA, DESCA, WORK( INDD ), WORK( INDE ),
     $              WORK( INDTAUQ ), WORK( INDTAUP ), WORK( INDWORK ),
     $              LLWORK, INFO )
*
* Copy D and E to all processes.
* Array D is in local array of dimension:
*  LOCc(JA+MIN(M,N)-1) if M >= N; LOCr(IA+MIN(M,N)-1) otherwise.
* Array E is in local array of dimension
*  LOCr(IA+MIN(M,N)-1) if M >= N; LOCc(JA+MIN(M,N)-2) otherwise.
*
      IF( M.GE.N ) THEN
* Distribute D
         CALL PDLARED1D( N+IOFFD, IA, JA, DESCA, WORK( INDD ),
     $                   WORK( INDD2 ), WORK( INDWORK ), LLWORK )
* Distribute E
         CALL PDLARED2D( M+IOFFE, IA, JA, DESCA, WORK( INDE ),
     $                   WORK( INDE2 ), WORK( INDWORK ), LLWORK )
      ELSE
* Distribute D
         CALL PDLARED2D( M+IOFFD, IA, JA, DESCA, WORK( INDD ),
     $                   WORK( INDD2 ), WORK( INDWORK ), LLWORK )
* Distribute E
         CALL PDLARED1D( N+IOFFE, IA, JA, DESCA, WORK( INDE ),
     $                   WORK( INDE2 ), WORK( INDWORK ), LLWORK )
      END IF
*
* Prepare for calling PDBDSQR.
*
      IF( M.GE.N ) THEN
         UPLO = 'U'
      ELSE
         UPLO = 'L'
      END IF
*
      CALL DBDSQR( UPLO, SIZE, 0, 0, 0, WORK( INDD2+IOFFD ),
     $             WORK( INDE2+IOFFE ), WORK( INDV ), SIZE,
     $             WORK( INDU ), 1, C, 1, WORK( INDWORK ), INFO )
*
* Copy singular values into output array S.
*
      DO 10 I = 1, SIZE
         S( I ) = WORK( INDD2+IOFFD+I-1 )
   10 CONTINUE
*
* If matrix was scaled, then rescale singular values appropriately.
*
      IF( ISCALE .EQ. 1 ) THEN
         CALL DSCAL( SIZE, ONE / SIGMA, S, 1 )
      END IF
*
* Compare every ith eigenvalue, or all if there are only a few,
* across the process grid to check for heterogeneity.
*
      IF( SIZE .LE. ITHVAL ) THEN
         J = SIZE
         K = 1
      ELSE
         J = SIZE / ITHVAL
         K = ITHVAL
      END IF
*
      DO 20 I = 1, J
         WORK( I+INDE ) = S( ( I-1 )*K+1 )
         WORK( I+INDD2 ) = S( ( I-1 )*K+1 )
   20 CONTINUE
*
      CALL DGAMN2D( DESCA( CTXT_ ), 'a', ' ', J, 1, WORK( 1+INDE ), J,
     $              1, 1, -1, -1, 0 )
      CALL DGAMX2D( DESCA( CTXT_ ), 'a', ' ', J, 1, WORK( 1+INDD2 ), J,
     $              1, 1, -1, -1, 0 )
*
      DO 30 I = 1, J
         IF( ( WORK( I+INDE )-WORK( I+INDD2 ) ).NE.ZERO ) THEN
            INFO = SIZE + 1
         END IF
   30 CONTINUE
*
   40 CONTINUE
*
* End of PDGESVD4
*
      RETURN
      END
